#include <cstdlib>
#include <memory>
#include <functional>
#include <iostream>
#include <cstring>

using namespace std;
using FT = function<void(char*)>; // function type alias of lambda operation

// lambda deleter
auto NoDelete = [](char *)  { };
auto Free     = [](char *p) { free(p); };
auto Delete   = [](const char *p) { delete p; };
auto ArrayDelete = [](const char *p) { delete[] p; };

struct MyDeleter {
	FT f{Delete};

	// mimic the default of unique_ptr, shared_ptr.
	MyDeleter() = default;
	// allow the user to specify what type of deleter to use
	explicit MyDeleter(FT f_) : f(move(f_)) {}
	void operator()(char *p) const { f(p); }
};

using uniquePtr = unique_ptr<char, MyDeleter>;

uniquePtr factory(int choice) {
	static char static_array[] = "Never ever delete this or undefined behavior will occur";
	switch (choice) {
		case 0:
			return { static_array, MyDeleter(NoDelete) };
		case 1: {
			uniquePtr p(new char, MyDeleter()); // default deleter
			*p = 'a';
			return p;
		}
		case 2: {
			uniquePtr p((char *) malloc(128), MyDeleter(Free));
			strcpy(p.get(), "malloc string");
			return p;
		}
		case 3: {
			uniquePtr p(new char[128], MyDeleter(ArrayDelete));
			strcpy(p.get(), "string allocated by new[]");
			return p;
		}
		default:
			return {nullptr, MyDeleter()}; // any deleter should work with null
	}
}

int main() {
	for (int i = 0; i < 4; ++i) {
		cout << factory(i).get() << endl;
	}
	return 0;
}

