#include <iostream>
#include <fstream>
#include <memory>
using namespace std;

void close_file(FILE* fp) {
	cout << "Closing file ... " << endl;
	fclose(fp);
}

// unique_ptr with deleter
void deal_with_resource(unique_ptr<FILE, decltype(&close_file)> fp) {
	constexpr int N{256};
	char str[N];
	if(fp) {
		cout << fgets(str, N, fp.get()) << endl;
	}
	// here fp goes out of scope & deleter is run
}

int main() {

	string file_name{"file.txt"};
	ofstream(file_name) << "some text written to external file ...";

	// something like Java 'try with resources'
	unique_ptr<FILE, decltype(&close_file)>
		fp(fopen(file_name.c_str(), "r"), &close_file);
	deal_with_resource(move(fp)); // move ptr to function & discard

	cout << "main() ends ..." << endl;
	return 0;
}

